SystemOrganization addCategory: #SpellingServices!
SystemOrganization addCategory: #'SpellingServices-Test'!

TestCase subclass: #SpellingServicesInterfaceTest
	instanceVariableNames: 'spelling'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SpellingServices-Test'!

!SpellingServicesInterfaceTest commentStamp: 'JMM 6/15/2005 16:53' prior: 0!
Mac spelling checking interface. Test cases
uses SpelllingPlugin
by
John M McIntosh
johnmci@smalltalkconsulting.com
June 15th 2005
!

SpellingServicesInterfaceTest subclass: #MacSpellingServicesInterfaceTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SpellingServices-Test'!

!MacSpellingServicesInterfaceTest methodsFor: 'running' stamp: 'tfel 2/14/2012 15:01:31.937'!
testATag
	| s |
	Smalltalk platformName asLowercase ~= 'mac os' ifTrue: [^ self].
	s := spelling primGetUniqueSpellingTag.
	self should: [s isNumber].
	self should: [s > 0].
	! !

!SpellingServicesInterfaceTest methodsFor: 'initialize-release' stamp: 'JMM 6/15/2005 09:48'!
setUp
	spelling _ MacSpellingInterface new.! !

!SpellingServicesInterfaceTest methodsFor: 'running' stamp: 'tfel 2/14/2012 15:02:08.659'!
testAGuess
	| s chk |

	s := spelling getGuesses: 'acklbar2'.
	self should: [s isEmpty].
	s := spelling getGuesses: 'recieve'.
	chk := s collect: [:e | e asString].
	self should: [chk includes: 'receive'].
	self should: [chk size > 1].

	! !

!SpellingServicesInterfaceTest methodsFor: 'running' stamp: 'JMM 6/15/2005 09:53'!
testGetIgnoredWords
	| s |
	s := spelling getIgnoredWords.
	self should: [s isCollection].
	! !

!SpellingServicesInterfaceTest methodsFor: 'running' stamp: 'JMM 6/15/2005 12:24'!
testGetSetLanguageString
	| l remember |
	l := spelling getLanguageString.
	self should: [l size > 0].
	remember _ l.
	spelling setLanguageString: l.
	l := spelling getLanguageString.
	self should: [l size > 0].
	self should: [remember = l].

	! !

!SpellingServicesInterfaceTest methodsFor: 'running' stamp: 'JMM 6/15/2005 16:56'!
testSetIgnoredWord
	|  w1 i2 i2strings w2 w3 |

	w1 := self waitASecondRollOver.
	spelling setIgnoredWord: w1.
	i2 := spelling getIgnoredWords.
	i2strings := i2 collect: [:e | e asString].
	self should: [i2strings includes: w1].

	w2 := self waitASecondRollOver.
	spelling setIgnoredWord: w2.
	i2 := spelling getIgnoredWords.
	i2strings := i2 collect: [:e | e asString].
	self should: [i2strings includes: w1].
	self should: [i2strings includes: w2].
	
	spelling setIgnoredWords: (OrderedCollection new).
	w3 := self waitASecondRollOver.
	spelling setIgnoredWord: w3.
	i2 := spelling getIgnoredWords.
	self should: [i2 size = 1].
	i2strings := i2 collect: [:e | e asString].
	self should: [i2strings includes: w3].

! !

!SpellingServicesInterfaceTest methodsFor: 'running' stamp: 'JMM 6/15/2005 16:38'!
testSetIgnoredWords
	|  w1 i i2 i3 i3strings i1strings i2strings |
	i := spelling getIgnoredWords.
	i isEmpty ifTrue: [
		spelling setIgnoredWords: (OrderedCollection with: 'ackbar').
		i2 := spelling getIgnoredWords.
		self should: [i2 size = 1].
		self should: [(i2 at: 1) asString = 'ackbar'].
		spelling setIgnoredWords: (OrderedCollection with: 'ackbar' with: 'ackbar2').
		i3 := spelling getIgnoredWords.
		self should: [i3 size = 2].
		i3strings := i3 collect: [:e | e asString].
		self should: [i3strings includes: 'ackbar'].
		self should: [i3strings includes: 'ackbar2']	
	].
	i := spelling getIgnoredWords.
	w1 := self waitASecondRollOver.
	i1strings := i collect: [:e | e asString].
	i1strings add: w1.
	spelling setIgnoredWords: i1strings.
	i2 := spelling getIgnoredWords.
	i2strings := i2 collect: [:e | e asString].
	self should: [i2strings includes: w1].
	spelling setIgnoredWords: (OrderedCollection new).
	i := spelling getIgnoredWords.
	self should: [i size = 0].


! !

!SpellingServicesInterfaceTest methodsFor: 'running' stamp: 'JMM 6/15/2005 16:48'!
testSpelling
	| s i |

	spelling setIgnoredWords: (OrderedCollection new).
	i := spelling getIgnoredWords.
	self should: [i size = 0].
	s := spelling checkSpellingOf: 'recieve' startingAt: 1.
	self should: [(s at: 1) = 1].
	self should: [(s at: 2) = 7].
	s := spelling checkSpellingOf: 'receive' startingAt: 1.
	self should: [(s at: 2) = 0].
	spelling setIgnoredWord: 'recieve'.
	s := spelling checkSpellingOf: 'recieve' startingAt: 1.
	self should: [(s at: 2) = 0].
	spelling setIgnoredWords: (OrderedCollection new).
	s := spelling checkSpellingOf: 'recieve' startingAt: 1.
	self should: [(s at: 1) = 1].
	self should: [(s at: 2) = 7].
	! !

!SpellingServicesInterfaceTest methodsFor: 'running' stamp: 'JMM 6/15/2005 16:59'!
testSpellingOffset
	| s i |

	spelling setIgnoredWords: (OrderedCollection new).
	i := spelling getIgnoredWords.
	self should: [i size = 0].
	s := spelling checkSpellingOf: 'too recieve' startingAt: 1.
	self should: [(s at: 1) = 5].
	self should: [(s at: 2) = 7].
	s := spelling checkSpellingOf: 'too more recieve' startingAt: 1.
	self should: [(s at: 1) = 10].
	self should: [(s at: 2) = 7].
	s := spelling checkSpellingOf: 'toox more recieve' startingAt: 1.
	self should: [(s at: 1) = 1].
	self should: [(s at: 2) = 4].
	s := spelling checkSpellingOf: 'toox more recieve' startingAt: 6.
	self should: [(s at: 1) = 11].
	self should: [(s at: 2) = 7].
! !

!SpellingServicesInterfaceTest methodsFor: 'utility' stamp: 'JMM 6/15/2005 14:16'!
waitASecondRollOver
	| now duration |
	now := Time  millisecondClockValue.
	[duration := (Time  millisecondClockValue) - now.
	duration > 0]
		whileFalse: [(Delay forMilliseconds: 10) wait].
	^Time  millisecondClockValue printString! !

Object subclass: #SpellingInterface
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SpellingServices'!

SpellingInterface subclass: #IspellSpellingInterface
	instanceVariableNames: ''
	classVariableNames: 'BinaryPath CalloutDict IgnoredWords LanguageString'
	poolDictionaries: ''
	category: 'SpellingServices'!

!IspellSpellingInterface class methodsFor: 'preferences' stamp: 'tfel 2/14/2012 14:19:06.895'!
binaryPath
	<preference: 'I/A/Spell binary path'
		category: 'Spelling Interface'
		description: 'Set the path of the binary to be used for spelling. Has to be ispell compatible.'
		type: #String>
	^ BinaryPath ifNil: ['aspell']
! !

!IspellSpellingInterface class methodsFor: 'preferences' stamp: 'tfel 2/14/2012 14:18:59.542'!
binaryPath: aString

	BinaryPath := aString.! !

!IspellSpellingInterface methodsFor: 'primitive-callouts' stamp: 'tfel 2/14/2012 14:13:04.362'!
apiLinuxLibc5Callout: aCommand
	<cdecl: long 'system' ( char* ) module: 'libc.so.5'>
	^ self externalCallFailed! !

!IspellSpellingInterface methodsFor: 'primitive-callouts' stamp: 'tfel 2/14/2012 14:13:13.257'!
apiLinuxLibc6Callout: aCommand
	<cdecl: long 'system' ( char* ) module: 'libc.so.6'>
	^ self externalCallFailed! !

!IspellSpellingInterface methodsFor: 'primitive-callouts' stamp: 'tfel 2/14/2012 14:13:36.297'!
apiMacOSXCallout: aCommand
	<cdecl: long 'system' (char*) module: 'libSystem.dylib'>
	| errCode |
	errCode := ExternalFunction getLastError.
	^ errCode = 13 "Unable to find function address. dylibs are not searched"
		ifTrue: [self error: 'Cannot call libSystem.dylib.
			Please set "SqueakPluginsBuiltInOrLocalOnly" in your VM''s Info.plist to false']
		ifFalse: [self externalCallFailed]! !

!IspellSpellingInterface methodsFor: 'primitive-callouts' stamp: 'tfel 2/14/2012 14:13:49.447'!
apiWindowsCallout: aCommand
	<apicall: long 'system' ( char* ) module: 'msvcrt.dll'>
	^ self externalCallFailed! !

!IspellSpellingInterface methodsFor: 'primitive-callouts' stamp: 'tfel 2/14/2012 14:14:45.396'!
callout: aCommand

	^ (Smalltalk classNamed: 'OSProcess')
		ifNil: [(self calloutDict
				at: SmalltalkImage current platformName asLowercase
				ifAbsent: [[:cmd | self error: 'Callouts not implemented for this platform!!']])
					value: aCommand]
		ifNotNilDo: [:class | class waitForCommand: aCommand].! !

!IspellSpellingInterface methodsFor: 'primitive-callouts' stamp: 'tfel 2/14/2012 14:15:40.678'!
calloutDict

	CalloutDict ifNil: [CalloutDict := Dictionary newFrom: {
										'unix' -> [:cmd | self linuxCallout: cmd].
										'win32' -> [:cmd | self windowsCallout: cmd].
										'mac os' -> [:cmd | self apiMacOSXCallout: cmd]}].
	^ CalloutDict! !

!IspellSpellingInterface methodsFor: 'spelling' stamp: 'tfel 2/14/2012 15:47:33.289'!
checkSpellingOf: aString startingAt: anInteger
	"returns a tuple, first element is start of first bad word, second is number of bad chars. If no incorrect word was found, returns #(0 0)"
	| words checkedString |
	checkedString := aString copyFrom: anInteger to: aString size.
	words := checkedString findTokens: Character separators, '-.:,?!!'.
	(self runIspellFor: checkedString) withIndexDo: [:line :idx |
		(line notEmpty and: [
		(line first = $&) or: [line first = $?] or: [line first = $#]]) "Miss, Guess, None"
			ifTrue: [| falseWord |
				falseWord := (words at: idx).
				^ { (checkedString findString: falseWord) + anInteger - 1.
					falseWord size }]].
	^ #(0 0)! !

!IspellSpellingInterface methodsFor: 'guesses' stamp: 'tfel 2/14/2012 15:08:17.582'!
getGuesses: aWord
	| idx line |
	line := (self runIspellFor: aWord) first "firstLine".
	^ (idx := line indexOf: $:) ~= 0 "colon separates guesses from other info"
		ifTrue: [(line copyFrom: idx to: line size) findTokens: ', ']
		ifFalse: [#()]
! !

!IspellSpellingInterface methodsFor: 'ignored words' stamp: 'tfel 2/14/2012 15:27:00.277'!
getIgnoredWords

	^ IgnoredWords ifNil: [IgnoredWords := OrderedCollection new]! !

!IspellSpellingInterface methodsFor: 'language' stamp: 'tfel 2/14/2012 15:28:19.456'!
getLanguageString

	^ LanguageString ifNil: [LanguageString := 'en_US.UTF-8']! !

!IspellSpellingInterface methodsFor: 'ignored words' stamp: 'tfel 2/14/2012 15:42:00.801'!
ignoredWordsForIspell

	^ String streamContents: [:stream |
		self getIgnoredWords do: [:word |
			stream
				nextPut: $@; "Accept-word-from-now-on-character"
				nextPutAll: word;
				lf]]! !

!IspellSpellingInterface methodsFor: 'primitive-callouts' stamp: 'tfel 2/14/2012 14:16:06.287'!
linuxCallout: aCommand
	"Try with new libc6 and older libc5"
	[self apiLinuxLibc6Callout: aCommand]
		on: Error
		do: [self apiLinuxLibc5Callout: aCommand].! !

!IspellSpellingInterface methodsFor: 'private-callouts' stamp: 'tfel 2/14/2012 15:41:54.911'!
runIspellFor: aString
	| file cmd fileName outputFileName result |
	fileName := self class name, '.txt'.
	outputFileName := self class name, 'Output.txt'.
	file := (FileDirectory default forceNewFileNamed: fileName).
	file
		nextPutAll: self ignoredWordsForIspell;
		nextPutAll: aString;
		lf;
		flush.
	cmd := String streamContents: [:stream |
		stream
			nextPutAll: 'cat ';
			nextPutAll: fileName;
			space;
			nextPut: $|;
			space;
			nextPut: $";
			nextPutAll: self class binaryPath;
			nextPut: $";
			nextPutAll: ' -a -f ';
			nextPutAll: outputFileName].
	self callout: cmd.
	FileDirectory default deleteFileNamed: fileName.
	result := (FileDirectory default fileNamed: outputFileName) contents.
	FileDirectory default deleteFileNamed: outputFileName.
	^ result lines allButFirst
	! !

!IspellSpellingInterface methodsFor: 'ignored words' stamp: 'tfel 2/14/2012 15:26:54.453'!
setIgnoredWord: aWord

	self getIgnoredWords add: aWord.! !

!IspellSpellingInterface methodsFor: 'ignored words' stamp: 'tfel 2/14/2012 15:26:23.955'!
setIgnoredWords: anArray

	IgnoredWords := anArray asOrderedCollection! !

!IspellSpellingInterface methodsFor: 'language' stamp: 'tfel 2/14/2012 15:25:47.457'!
setLanguageString: aString

	LanguageString := aString
! !

!IspellSpellingInterface methodsFor: 'primitive-callouts' stamp: 'tfel 2/14/2012 14:16:34.954'!
windowsCallout: aCommand
	"Escape newlines for Windows CmdLine"
	| escapedCommand |
	escapedCommand := ((aCommand
								copyReplaceAll: Character cr asString with: ' ')
								copyReplaceAll: Character lf asString with: ' ').
	self apiWindowsCallout: escapedCommand.! !

SpellingInterface subclass: #MacSpellingInterface
	instanceVariableNames: ''
	classVariableNames: 'Tag'
	poolDictionaries: ''
	category: 'SpellingServices'!

!MacSpellingInterface class methodsFor: 'instance creation' stamp: 'tfel 2/14/2012 15:49:54.838'!
basicNew

	^ [super basicNew
		primGetLanguage;
		yourself] on: Error do: [
			"Spelling plugin seems to be unavailable, use Ispell"
			IspellSpellingInterface basicNew]! !

!MacSpellingInterface methodsFor: 'spelling' stamp: 'JMM 6/15/2005 09:43'!
checkSpellingOf: aString startingAt: anInteger
	| results utf16 |

	utf16 := self unknownToUTF16: aString.
	results := self primCheckSpelling: utf16 startingAt: anInteger.
	^results! !

!MacSpellingInterface methodsFor: 'guesses' stamp: 'JMM 6/15/2005 09:44'!
getGuesses: aString
	| size collection word utf16 toGuess |
	
	toGuess := self unknownToUTF16: aString.
	size := self primGetGuessesForWordListLength: toGuess withTag: Tag.
	collection := OrderedCollection new: size.
	1 to: size do: [:i |
		word := self primGetGuessesForWordwithTag: Tag at: i.
		utf16 := self byteStringToUTF16: word.	
		collection add: utf16].
	^collection! !

!MacSpellingInterface methodsFor: 'ignored words' stamp: 'JMM 6/15/2005 09:36'!
getIgnoredWords
	| size collection word utf16 |
	
	size := self primGetIgnoredWordsListLengthWithTag: Tag.
	size isZero ifTrue: [^OrderedCollection new].
	collection := OrderedCollection new: size.
	1 to: size do: [:i |
		word := self primGetIgnoredWordWithTag: Tag at: i.
		utf16 := self byteStringToUTF16: word.
		collection add: utf16].
	^collection! !

!MacSpellingInterface methodsFor: 'language' stamp: 'JMM 6/14/2005 18:37'!
getLanguageString
	"ByteString , ie english"

	^self primGetLanguage! !

!MacSpellingInterface methodsFor: 'initialize-release' stamp: 'JMM 6/14/2005 17:59'!
initialize
	Tag := self primGetUniqueSpellingTag.

! !

!MacSpellingInterface methodsFor: 'system primitives' stamp: 'JMM 6/14/2005 18:18'!
primCheckSpelling: aString startingAt: anInteger

	<primitive:'primitiveCheckSpelling' module: 'SpellingPlugin'>
! !

!MacSpellingInterface methodsFor: 'system primitives' stamp: 'JMM 6/14/2005 18:14'!
primGetGuessesForWordListLength: data withTag: aTag

	<primitive:'primitiveGetGuessesForWordListLength' module: 'SpellingPlugin'>
! !

!MacSpellingInterface methodsFor: 'system primitives' stamp: 'JMM 6/14/2005 18:14'!
primGetGuessesForWordwithTag: aTag at: index

	<primitive:'primitiveGetGuessesForWordwithTag' module: 'SpellingPlugin'>
! !

!MacSpellingInterface methodsFor: 'system primitives' stamp: 'JMM 6/14/2005 18:12'!
primGetIgnoredWordWithTag: Tag at: i

	<primitive:'primitiveGetIgnoredWordWithTag' module: 'SpellingPlugin'>
! !

!MacSpellingInterface methodsFor: 'system primitives' stamp: 'JMM 6/14/2005 18:10'!
primGetIgnoredWordsListLengthWithTag: aTag

	<primitive:'primitiveGetIgnoredWordsListLengthWithTag' module: 'SpellingPlugin'>
! !

!MacSpellingInterface methodsFor: 'system primitives' stamp: 'JMM 6/14/2005 18:07'!
primGetLanguage

	<primitive:'primitiveGetLanguage' module: 'SpellingPlugin'>
! !

!MacSpellingInterface methodsFor: 'system primitives' stamp: 'JMM 6/14/2005 17:59'!
primGetUniqueSpellingTag

	<primitive:'primitiveGetUniqueSpellingTag' module: 'SpellingPlugin'>
! !

!MacSpellingInterface methodsFor: 'system primitives' stamp: 'JMM 6/14/2005 18:04'!
primSetIgnoredWords: string withTag: Tag at: i

	<primitive:'primitiveSetIgnoredWords' module: 'SpellingPlugin'>
! !

!MacSpellingInterface methodsFor: 'system primitives' stamp: 'JMM 6/14/2005 18:07'!
primSetIgnoredWordsListLength: anArray withTag: Tag

	<primitive:'primitiveSetIgnoredWordsListLength' module: 'SpellingPlugin'>
! !

!MacSpellingInterface methodsFor: 'system primitives' stamp: 'JMM 6/14/2005 18:08'!
primSetLanguage: aString

	<primitive:'primitiveSetLanguage' module: 'SpellingPlugin'>
! !

!MacSpellingInterface methodsFor: 'system primitives' stamp: 'JMM 6/14/2005 18:15'!
primSetNewIgnoredWord: aString withTag: Tag

	<primitive:'primitiveSetNewIgnoredWord' module: 'SpellingPlugin'>
! !

!MacSpellingInterface methodsFor: 'ignored words' stamp: 'JMM 6/15/2005 09:39'!
setIgnoredWord: aString
	"unicode32 is unicode value"
	| utf16 |

	utf16 := self unknownToUTF16: aString.
	self primSetNewIgnoredWord: utf16 withTag: Tag
! !

!MacSpellingInterface methodsFor: 'ignored words' stamp: 'JMM 6/15/2005 09:40'!
setIgnoredWords: anArray
	"AnArray is unicode 16bit values"

	| utf16 |
	self primSetIgnoredWordsListLength: anArray size withTag: Tag.
	1 to: anArray size  do: [:i | 
		utf16 _ self unknownToUTF16: (anArray at: i).
		self primSetIgnoredWords: utf16 withTag: Tag at: i].! !

!MacSpellingInterface methodsFor: 'language' stamp: 'JMM 6/14/2005 18:21'!
setLanguageString: aString
	"ByteString , ie english"
	self primSetLanguage: aString! !

!SpellingInterface class methodsFor: 'instance creation' stamp: 'tfel 2/14/2012 15:05:59.601'!
new

	^ (SmalltalkImage current platformName asLowercase = 'mac os'
		ifTrue: [MacSpellingInterface]
		ifFalse: [IspellSpellingInterface])
			basicNew
			initialize;
			yourself! !

!SpellingInterface methodsFor: 'converter' stamp: 'tfel 2/14/2012 13:59:56.304'!
byteStringToUTF16: bytes

	^bytes asString convertFromWithConverter: (TextConverter newForEncoding: 'utf-16').
! !

!SpellingInterface methodsFor: 'spelling' stamp: 'tfel 2/14/2012 14:03:44.403'!
checkSpellingOf: aString startingAt: anInteger

	self subclassResponsibility.! !

!SpellingInterface methodsFor: 'guesses' stamp: 'tfel 2/14/2012 14:00:23.17'!
getGuesses: aString

	self subclassResponsibility.! !

!SpellingInterface methodsFor: 'ignored words' stamp: 'tfel 2/14/2012 14:00:57.406'!
getIgnoredWords

	self subclassResponsibility.! !

!SpellingInterface methodsFor: 'language' stamp: 'tfel 2/14/2012 14:03:25.881'!
getLanguageString

	self subclassResponsibility.! !

!SpellingInterface methodsFor: 'ignored words' stamp: 'tfel 2/14/2012 14:01:12.745'!
setIgnoredWord: aString

	self subclassResponsibility.! !

!SpellingInterface methodsFor: 'ignored words' stamp: 'tfel 2/14/2012 15:20:59.32'!
setIgnoredWords: anArray
	
	self subclassResponsibility.
! !

!SpellingInterface methodsFor: 'language' stamp: 'tfel 2/14/2012 14:03:33.861'!
setLanguageString: aString

	self subclassResponsibility.! !

!SpellingInterface methodsFor: 'converter' stamp: 'tfel 2/14/2012 13:59:51.129'!
unknownToUTF16: candidate

	^candidate convertToWithConverter: (TextConverter newForEncoding: 'utf-16').
! !
